require "Window"
require "MatchingGame"
 
local KillingBlow = {} 
local GeminiColor
local ktClassSprites = {
	[GameLib.CodeEnumClass.Warrior] 			= "ClientSprites:Icon_Windows_UI_CRB_Warrior",
	[GameLib.CodeEnumClass.Engineer] 			= "ClientSprites:Icon_Windows_UI_CRB_Engineer",
	[GameLib.CodeEnumClass.Esper]				= "ClientSprites:Icon_Windows_UI_CRB_Esper",
	[GameLib.CodeEnumClass.Medic]				= "ClientSprites:Icon_Windows_UI_CRB_Medic",
	[GameLib.CodeEnumClass.Stalker] 			= "ClientSprites:Icon_Windows_UI_CRB_Stalker",
	[GameLib.CodeEnumClass.Spellslinger]	 	= "ClientSprites:Icon_Windows_UI_CRB_Spellslinger"
}
local ktClassColors = { -- Default Colors from: http://www.wildstar-online.com/en/#classes
	[GameLib.CodeEnumClass.Warrior] 			= "ffff0000", -- Red
	[GameLib.CodeEnumClass.Engineer] 			= "ffff4500", -- Orange
	[GameLib.CodeEnumClass.Esper]				= "ff1e90ff", -- Blue
	[GameLib.CodeEnumClass.Medic]				= "ffffff00", -- Yellow
	[GameLib.CodeEnumClass.Stalker] 			= "ff7b68ee", -- Purple
	[GameLib.CodeEnumClass.Spellslinger]	 	= "ff00ff00"  -- Green
}
local fonts = {
	{ sDisplayName = "Heading", 				sFontName = "CRB_Dialog_Heading_Huge" },
	{ sDisplayName = "Floater Large", 			sFontName = "CRB_FloaterGigantic" },
	{ sDisplayName = "Floater Large Outlined", 	sFontName = "CRB_FloaterGigantic_O" },
	{ sDisplayName = "Floater Medium", 			sFontName = "CRB_FloaterMedium" },
	{ sDisplayName = "Subtitle", 				sFontName = "Subtitle" },
}

local kstrCreatureSprite = "CRB_GuildSprites:sprGuild_Skull"
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function KillingBlow:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	o.config = {
		bAlertLock = false,
		bKillSound = true,
		bChatLog = true,
		sKBColor = "ffff0000",
		sPlayerColor = "ffffffff",
		sKBFont = "CRB_FloaterLarge",
		sPlayerFont = "CRB_FloaterLarge",
	}

    return o
end

function KillingBlow:Init()
    Apollo.RegisterAddon(self, true, "KillingBlow", {"GeminiColor"})
end
 

-----------------------------------------------------------------------------------------------
-- KillingBlow OnLoad
-----------------------------------------------------------------------------------------------
function KillingBlow:OnLoad()
	GeminiColor = Apollo.GetPackage("GeminiColor").tPackage

    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("KillingBlow.xml")
	self.xmlSettingsDoc = XmlDoc.CreateFromFile("Settings.xml")
	
	if not self.xmlDoc or not self.xmlSettingsDoc then
		Apollo.AddAddonErrorText(self, "Unable to load required xml files. Installation corrupt!")
		return
	end
		
	if GameLib.GetPlayerUnit() then
		self:OnDocLoaded()
	else
		Apollo.RegisterEventHandler("CharacterCreated", "OnDocLoaded", self)
	end
end

-----------------------------------------------------------------------------------------------
-- KillingBlow OnDocLoaded
-----------------------------------------------------------------------------------------------
function KillingBlow:OnDocLoaded()
	-- Events
	Apollo.RegisterEventHandler("PvpKillNotification", "PvpKillEvent", self)
	Apollo.RegisterEventHandler("MatchEntered", "OnPVPMatchEntered", self)
	Apollo.RegisterEventHandler("MatchExited", "HideAlert", self)
	
	-- Slash Commands
	Apollo.RegisterSlashCommand("kbshow", "OnShowSlashCommand", self)
	
	-- Timers
	Apollo.RegisterTimerHandler("Alert", "HideAlert", self)

	if MatchingGame:IsInMatchingGame() then
		local tMatchState = MatchingGame:GetPVPMatchState()
		if tMatchState ~= nil then
			self:OnPVPMatchEntered()
		end
	end
end

-----------------------------------------------------------------------------------------------
-- KillingBlow OnShowSlashCommand
-- Command: "/kbshow"
-----------------------------------------------------------------------------------------------
function KillingBlow:OnShowSlashCommand()
	self:LoadAllFrames()
	
	local drPlayer = GameLib.GetPlayerUnit()
	local strPName = drPlayer:GetName()
	local eClass = drPlayer:GetClassId()
	
	-- Send test event and open the settings window
	self:PvpKillEvent(strPName, MatchingGame.PvpDeathReason.KilledByPlayer, strPName, eClass, MatchingGame.Team.Team1, true)
	self.wndKBSettings:Show(true)
	self.wndKBSettingsOptions:Show(true)
	
	self:LoadOptions()
end

-----------------------------------------------------------------------------------------------
-- KillingBlow LoadOptions
-----------------------------------------------------------------------------------------------
function KillingBlow:LoadOptions()
	-- General Options
	local wndGeneralOptions = self.wndKBSettingsOptions:FindChild("General")

	-- Lock
	wndGeneralOptions:FindChild("LockAlertWindow"):SetCheck(self.config.bAlertLock)
	self.wndKBAlert:SetStyle("Moveable", not self.config.bAlertLock)
	self.wndKBAlert:SetStyle("IgnoreMouse", self.config.bAlertLock)
	
	-- Log
	wndGeneralOptions:FindChild("LogToChat"):SetCheck(self.config.bChatLog)
	
	-- Sound
	wndGeneralOptions:FindChild("EnableSound"):SetCheck(self.config.bKillSound)
	
	
	-- Text Options
	self:UpdateKBColor(self.config.sKBColor)
	self:UpdatePlayerColor(self.config.sPlayerColor)

	self:SetKBFont(self.config.sKBFont)
	self:SetPlayerFont(self.config.sPlayerFont)
end

-----------------------------------------------------------------------------------------------
-- KillingBlow OnSave
-----------------------------------------------------------------------------------------------
function KillingBlow:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	-- Save current window position
	local l,t,r,b = self.wndKBAlert:GetAnchorOffsets()
	self.config.tPosition = {l = l, t = t, r = r, b = b}
	
	return self.config
end

-----------------------------------------------------------------------------------------------
-- KillingBlow OnRestore
-----------------------------------------------------------------------------------------------
function KillingBlow:OnRestore(eType, tData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	-- Load all frames before updating with restored data
	self:LoadAllFrames()
	
	-- Load saved window position
	local tPos = tData.tPosition
	if tPos ~= nil then
		self.wndKBAlert:SetAnchorOffsets(tPos.l, tPos.t, tPos.r, tPos.b)
	end
	
	-- Window lock
	if tData.bAlertLock ~= nil then
		self.config.bAlertLock = tData.bAlertLock
	end
	-- Log options
	if tData.bChatLog ~= nil then
		self.config.bChatLog = tData.bChatLog
	end
	-- Sound options
	if tData.bKillSound ~= nil then
		self.config.bKillSound = tData.bKillSound
	end
	-- KB color
	if tData.sKBColor ~= nil then
		self.config.sKBColor = tData.sKBColor
	end
	-- Player color
	if tData.sPlayerColor ~= nil then
		self.config.sPlayerColor = tData.sPlayerColor
	end
	-- KB font
	if tData.sKBFont ~= nil then
		self.config.sKBFont = tData.sKBFont
	end
	-- Player font
	if tData.sPlayerFont ~= nil then
		self.config.sPlayerFont = tData.sPlayerFont 
	end
		
	self:LoadOptions()
end

-----------------------------------------------------------------------------------------------
-- KillingBlow HideSettings
-----------------------------------------------------------------------------------------------
function KillingBlow:HideSettings()
	if self.wndKBSettings:IsValid() then
		self.wndKBSettings:Destroy()
	end

	self:HideAlert()
end

-----------------------------------------------------------------------------------------------
-- KillingBlow HideAlert
-----------------------------------------------------------------------------------------------
function KillingBlow:HideAlert()
	if self.wndKBAlert:IsShown() then
		self.wndKBAlert:Show(false)
	end
end

-----------------------------------------------------------------------------------------------
-- KillingBlow SetAlertMoveable
-- Lock Position Checkbox
-----------------------------------------------------------------------------------------------
function KillingBlow:SetAlertMoveable(wndHandler, wndControl, eMouseButton)
	self.config.bAlertLock = wndControl:IsChecked()
	self:LoadOptions()
end

-----------------------------------------------------------------------------------------------
-- KillingBlow SetLogOnOff
-- Log To Chat Checkbox
-----------------------------------------------------------------------------------------------
function KillingBlow:SetLogOnOff(wndHandler, wndControl, eMouseButton)
	self.config.bChatLog = wndControl:IsChecked()
end

-----------------------------------------------------------------------------------------------
-- KillingBlow SetSoundOnOff
-- Log To Chat Checkbox
-----------------------------------------------------------------------------------------------
function KillingBlow:SetSoundOnOff(wndHandler, wndControl, eMouseButton)
	self.config.bKillSound = wndControl:IsChecked()
end

-----------------------------------------------------------------------------------------------
-- KillingBlow PvpKillEvent
-----------------------------------------------------------------------------------------------
function KillingBlow:PvpKillEvent(strVictimName, eReason, strKillerName, eKillerClass, eVictimTeam, bSlash)

	if not self.wndKBAlert then
		return
	end
	
	-- Current Player
	local drPlayer = GameLib.GetPlayerUnit()
	local strPName = drPlayer:GetName()
	
	
	-- If the current player did not kill the unit do not display
	if strKillerName ~= strPName then
		return
	end
	
	-- Load the victims name, class name, class icon
	local drVictim = GameLib.GetPlayerUnitByName(strVictimName)
	local ktSprite = self:GetClassSprite(drVictim)
	local stVictimClass = self:GetClassName(drVictim)
	
	
	self.wndKBAlert:FindChild("PlayerNameText"):SetText(strVictimName)
	self.wndKBAlert:FindChild("ClassIcon"):SetSprite(ktSprite)
	
	-- Display the killing blow
	self.wndKBAlert:Show(true)
		
	-- If the slash command was used to display don't use the timer
	if not bSlash then
		self:LogKill(stVictimClass, strVictimName)
		self:PlayKillSound()
		Apollo.CreateTimer("Alert", 3.5, false)
	end
end

-----------------------------------------------------------------------------------------------
-- KillingBlow LogKill
-----------------------------------------------------------------------------------------------
function KillingBlow:LogKill(stVictimClass, strVictimName) 
	if self.config.bChatLog then
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Killing Blow! " .. stVictimClass .. " " .. strVictimName)
	end
end

function KillingBlow:PlayKillSound()
	if self.config.bKillSound then
		Sound.Play(198)
	end
end

-----------------------------------------------------------------------------------------------
-- KillingBlow OnPVPMatchEntered
-----------------------------------------------------------------------------------------------
function KillingBlow:OnPVPMatchEntered()
	if self:InPVP() then
		self:LoadAlertFrame()
	end
end

-----------------------------------------------------------------------------------------------
-- KillingBlow LoadAllFrames
-----------------------------------------------------------------------------------------------
function KillingBlow:LoadAllFrames()
	self:LoadAlertFrame()
	self:LoadSettingsFrame()
end

-----------------------------------------------------------------------------------------------
-- KillingBlow LoadAlertFrame
-----------------------------------------------------------------------------------------------
function KillingBlow:LoadAlertFrame()
	if self.wndKBAlert and self.wndKBAlert:IsValid() then
		return
	end
	
	self.wndKBAlert = Apollo.LoadForm(self.xmlDoc, "KillingBlowAlert", nil, self)
	if self.wndKBAlert == nil then
		Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
		return
	end
	
	self.wndKBAlert:Show(false)
end

-----------------------------------------------------------------------------------------------
-- KillingBlow LoadSettingsFrame
-----------------------------------------------------------------------------------------------
function KillingBlow:LoadSettingsFrame()
	if self.wndKBSettings and self.wndKBSettings:IsValid() then
		return
	end
	
	self.wndKBSettings = Apollo.LoadForm(self.xmlSettingsDoc, "SettingsForm", nil, self)
	if self.wndKBSettings == nil then
		Apollo.AddAddonErrorText(self, "Could not load the settings window for some reason.")
		return
	end
	
	self.wndKBSettingsOptions = Apollo.LoadForm(self.xmlSettingsDoc, "Options", self.wndKBSettings:FindChild("ScrollFrame"), self)
	self.wndKBSettingsOptions:FindChild("Text"):FindChild("KBColorSwatch"):SetBGColor(self.config.sKBColor)

	self.wndKBSettings:Show(false)
end

-----------------------------------------------------------------------------------------------
-- KillingBlow InPVP
-----------------------------------------------------------------------------------------------
function KillingBlow:InPVP()
	if MatchingGame.IsInMatchingGame() or MatchingGame.IsInPVPGame() then
		return true
	end
	return false
end

-----------------------------------------------------------------------------------------------
-- KillingBlow GetClassSprite
-----------------------------------------------------------------------------------------------
function KillingBlow:GetClassSprite(drPlayer)
	-- If the victim couldn't be found use a default icon
	if drPlayer then
		return ktClassSprites[drPlayer:GetClassId()]
	end
	
	return kstrCreatureSprite
end

-----------------------------------------------------------------------------------------------
-- KillingBlow GetClassName
-----------------------------------------------------------------------------------------------
function KillingBlow:GetClassName(drPlayer)
	-- If the class coudln't be found return blank string
	if drPlayer then
		for k, v in pairs(GameLib.CodeEnumClass) do
			if v == drPlayer:GetClassId() then
				return k
			end
		end
	end
	return ""
end

-----------------------------------------------------------------------------------------------
-- KillingBlow OnKBColorMouseUp
-----------------------------------------------------------------------------------------------
function KillingBlow:OnKBColorMouseUp( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY )
	if wndHandler ~= wndControl then return end
	GeminiColor:ShowColorPicker(self, "UpdateKBColor", true, self.config.sKBColor)
end

function KillingBlow:UpdateKBColor(sColor)
	self.config.sKBColor = sColor
	if self.wndKBSettingsOptions and self.wndKBAlert then
		self.wndKBSettingsOptions:FindChild("Text"):FindChild("KBColorSwatch"):SetBGColor(sColor)
		self.wndKBAlert:FindChild("KillingBlowText"):SetTextColor(sColor)
	end
end

function KillingBlow:OnPlayerColorMouseUp( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY )
	if wndHandler ~= wndControl then return end
	GeminiColor:ShowColorPicker(self, "UpdatePlayerColor", true, self.config.sPlayerColor)
end

function KillingBlow:UpdatePlayerColor(sColor)
	self.config.sPlayerColor = sColor
	if self.wndKBSettingsOptions and self.wndKBAlert then
		self.wndKBSettingsOptions:FindChild("Text"):FindChild("PlayerColorSwatch"):SetBGColor(sColor)
		self.wndKBAlert:FindChild("PlayerNameText"):SetTextColor(sColor)
	end
end

function KillingBlow:OnKBFontToggle( wndHandler, wndControl, eMouseButton )
	if not self.wndKBFontContainer then
		self.wndKBFontContainer = wndHandler:FindChild("ChoiceContainer")
	end
	
	if not self.wndKBFontContainer:IsShown() then
		self.wndKBFontContainer:Show(true)
	else
		self.wndKBFontContainer:Show(false)
	end	
	
end

function KillingBlow:OnKBFontRadio( wndHandler, wndControl, eMouseButton )
	self.config.sKBFont = wndHandler:GetName()
	self:SetKBFont(self.config.sKBFont)
end

function KillingBlow:SetKBFont(sFont)
	if self.wndKBSettingsOptions and self.wndKBAlert then
		self.wndKBSettingsOptions:FindChild("Text"):FindChild("DropToggleKBFont"):SetFont(sFont)
		self.wndKBAlert:FindChild("KillingBlowText"):SetFont(sFont)
	end
end

function KillingBlow:OnPlayerFontToggle( wndHandler, wndControl, eMouseButton )
	if not self.wndPlayerFontContainer then
		self.wndPlayerFontContainer = wndHandler:FindChild("ChoiceContainer")
	end
	
	if not self.wndPlayerFontContainer:IsShown() then
		self.wndPlayerFontContainer:Show(true)
	else
		self.wndPlayerFontContainer:Show(false)
	end	
end

function KillingBlow:OnPlayerFontRadio( wndHandler, wndControl, eMouseButton )
	self.config.sPlayerFont = wndHandler:GetName()
	self:SetPlayerFont(self.config.sPlayerFont)
end

function KillingBlow:SetPlayerFont(sFont)
	if self.wndKBSettingsOptions and self.wndKBAlert then
		self.wndKBSettingsOptions:FindChild("Text"):FindChild("DropTogglePlayerFont"):SetFont(sFont)
		self.wndKBAlert:FindChild("PlayerNameText"):SetFont(sFont)
	end
end

--[[

	Utility Functions

]]--

-----------------------------------------------------------------------------------------------
-- KillingBlow BoolToString
-----------------------------------------------------------------------------------------------
function KillingBlow:BoolToString(b)
	return string.format("%s", b and "true" or "false")
end

-----------------------------------------------------------------------------------------------
-- KillingBlow Debug
-----------------------------------------------------------------------------------------------
function KillingBlow:Debug(s)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Debug, s)
end

-----------------------------------------------------------------------------------------------
-- KillingBlow Instance
-----------------------------------------------------------------------------------------------
local KillingBlowInst = KillingBlow:new()
KillingBlowInst:Init()
